@@ -1,4 +1,4 @@
-/* Copyright (C) 1995, 96, 97, 98, 99, 2000, 2002 Free Software Foundation, Inc.
+/* Copyright (C) 1995, 96, 97, 98, 99, 2000, 2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,7 +41,6 @@
 
 
 
-#include <stdio.h>
 #include "libguile/_scm.h"
 #include "libguile/eq.h"
 #include "libguile/list.h"
@@ -53,7 +52,7 @@
 
 SCM_DEFINE (scm_acons, "acons", 3, 0, 0,
            (SCM key, SCM value, SCM alist),
-	    "Adds a new key-value pair to @var{alist}.  A new pair is\n"
+	    "Add a new key-value pair to @var{alist}.  A new pair is\n"
 	    "created whose car is @var{key} and whose cdr is @var{value}, and the\n"
 	    "pair is consed onto @var{alist}, and the new list is returned.  This\n"
 	    "function is @emph{not} destructive; @var{alist} is not modified.")
@@ -134,25 +133,28 @@ SCM_DEFINE (scm_sloppy_assoc, "sloppy-assoc", 2, 0, 0,
 
 SCM_DEFINE (scm_assq, "assq", 2, 0, 0,
            (SCM key, SCM alist),
-	    "@deffnx primitive assv key alist\n"
-	    "@deffnx primitive assoc key alist\n"
-	    "Fetches the entry in @var{alist} that is associated with @var{key}.  To\n"
+	    "@deffnx {Scheme Procedure} assv key alist\n"
+	    "@deffnx {Scheme Procedure} assoc key alist\n"
+	    "Fetch the entry in @var{alist} that is associated with @var{key}.  To\n"
 	    "decide whether the argument @var{key} matches a particular entry in\n"
 	    "@var{alist}, @code{assq} compares keys with @code{eq?}, @code{assv}\n"
 	    "uses @code{eqv?} and @code{assoc} uses @code{equal?}.  If @var{key}\n"
 	    "cannot be found in @var{alist} (according to whichever equality\n"
-	    "predicate is in use), then @code{#f} is returned.  These functions\n"
+	    "predicate is in use), then return @code{#f}.  These functions\n"
 	    "return the entire alist entry found (i.e. both the key and the value).")
 #define FUNC_NAME s_scm_assq
 {
-  for (; SCM_CONSP (alist); alist = SCM_CDR (alist)) 
+  SCM ls = alist;
+  for (; SCM_CONSP (ls); ls = SCM_CDR (ls)) 
     {
-      SCM tmp = SCM_CAR (alist);
-      SCM_VALIDATE_CONS (SCM_ARG2, tmp);
+      SCM tmp = SCM_CAR (ls);
+      SCM_ASSERT_TYPE (SCM_CONSP (tmp), alist, SCM_ARG2, FUNC_NAME,
+		       "association list");
       if (SCM_EQ_P (SCM_CAR (tmp), key))
 	return tmp;
     }
-  SCM_VALIDATE_NULL (2, alist);
+  SCM_ASSERT_TYPE (SCM_NULLP (ls), alist, SCM_ARG2, FUNC_NAME,
+		   "association list");
   return SCM_BOOL_F;
 }
 #undef FUNC_NAME
@@ -163,14 +165,17 @@ SCM_DEFINE (scm_assv, "assv", 2, 0, 0,
 	    "Behaves like @code{assq} but uses @code{eqv?} for key comparison.")
 #define FUNC_NAME s_scm_assv
 {
-  for(; SCM_CONSP (alist); alist = SCM_CDR (alist)) 
+  SCM ls = alist;
+  for(; SCM_CONSP (ls); ls = SCM_CDR (ls)) 
     {
-      SCM tmp = SCM_CAR (alist);
-      SCM_VALIDATE_CONS (SCM_ARG2, tmp);
+      SCM tmp = SCM_CAR (ls);
+      SCM_ASSERT_TYPE (SCM_CONSP (tmp), alist, SCM_ARG2, FUNC_NAME,
+		       "association list");
       if (SCM_NFALSEP (scm_eqv_p (SCM_CAR (tmp), key)))
 	return tmp;
     }
-  SCM_VALIDATE_NULL (2, alist);
+  SCM_ASSERT_TYPE (SCM_NULLP (ls), alist, SCM_ARG2, FUNC_NAME,
+		   "association list");
   return SCM_BOOL_F;
 }
 #undef FUNC_NAME
@@ -181,14 +186,17 @@ SCM_DEFINE (scm_assoc, "assoc", 2, 0, 0,
 	    "Behaves like @code{assq} but uses @code{equal?} for key comparison.")
 #define FUNC_NAME s_scm_assoc
 {
-  for(; SCM_CONSP (alist); alist = SCM_CDR (alist)) 
+  SCM ls = alist;
+  for(; SCM_CONSP (ls); ls = SCM_CDR (ls)) 
     {
-      SCM tmp = SCM_CAR (alist);
-      SCM_VALIDATE_CONS (SCM_ARG2, tmp);
+      SCM tmp = SCM_CAR (ls);
+      SCM_ASSERT_TYPE (SCM_CONSP (tmp), alist, SCM_ARG2, FUNC_NAME,
+		       "association list");
       if (SCM_NFALSEP (scm_equal_p (SCM_CAR (tmp), key)))
 	return tmp;
     }
-  SCM_VALIDATE_NULL (2, alist);
+  SCM_ASSERT_TYPE (SCM_NULLP (ls), alist, SCM_ARG2, FUNC_NAME,
+		   "association list");
   return SCM_BOOL_F;
 }
 #undef FUNC_NAME
@@ -196,10 +204,22 @@ SCM_DEFINE (scm_assoc, "assoc", 2, 0, 0,
 
 
 
+/* Dirk:API2.0:: We should not return #f if the key was not found.  In the
+ * current solution we can not distinguish between finding a (key . #f) pair
+ * and not finding the key at all.
+ *
+ * Possible alternative solutions:
+ * 1) Remove assq-ref from the API:  assq is sufficient.
+ * 2) Signal an error (what error type?) if the key is not found.
+ * 3) provide an additional 'default' parameter.
+ * 3.1) The default parameter is mandatory.
+ * 3.2) The default parameter is optional, but if no default is given and
+ *      the key is not found, signal an error (what error type?).
+ */
 SCM_DEFINE (scm_assq_ref, "assq-ref", 2, 0, 0,
             (SCM alist, SCM key),
-	    "@deffnx primitive assv-ref alist key\n"
-	    "@deffnx primitive assoc-ref alist key\n"
+	    "@deffnx {Scheme Procedure} assv-ref alist key\n"
+	    "@deffnx {Scheme Procedure} assoc-ref alist key\n"
 	    "Like @code{assq}, @code{assv} and @code{assoc}, except that only the\n"
 	    "value associated with @var{key} in @var{alist} is returned.  These\n"
 	    "functions are equivalent to\n\n"
@@ -262,8 +282,8 @@ SCM_DEFINE (scm_assoc_ref, "assoc-ref", 2, 0, 0,
 
 SCM_DEFINE (scm_assq_set_x, "assq-set!", 3, 0, 0,
             (SCM alist, SCM key, SCM val),
-	    "@deffnx primitive assv-set! alist key value\n"
-	    "@deffnx primitive assoc-set! alist key value\n"
+	    "@deffnx {Scheme Procedure} assv-set! alist key value\n"
+	    "@deffnx {Scheme Procedure} assoc-set! alist key value\n"
 	    "Reassociate @var{key} in @var{alist} with @var{value}: find any existing\n"
 	    "@var{alist} entry for @var{key} and associate it with the new\n"
 	    "@var{value}.  If @var{alist} does not contain an entry for @var{key},\n"
@@ -327,9 +347,9 @@ SCM_DEFINE (scm_assoc_set_x, "assoc-set!", 3, 0, 0,
 
 SCM_DEFINE (scm_assq_remove_x, "assq-remove!", 2, 0, 0,
             (SCM alist, SCM key),
-	    "@deffnx primitive assv-remove! alist key\n"
-	    "@deffnx primitive assoc-remove! alist key\n"
-	    "Delete any entry in @var{alist} associated with @var{key}, and return\n"
+	    "@deffnx {Scheme Procedure} assv-remove! alist key\n"
+	    "@deffnx {Scheme Procedure} assoc-remove! alist key\n"
+	    "Delete the first entry in @var{alist} associated with @var{key}, and return\n"
 	    "the resulting alist.")
 #define FUNC_NAME s_scm_assq_remove_x
 {
@@ -337,11 +357,9 @@ SCM_DEFINE (scm_assq_remove_x, "assq-remove!", 2, 0, 0,
 
   handle = scm_sloppy_assq (key, alist);
   if (SCM_CONSP (handle))
-    {
-      return scm_delq_x (handle, alist);
-    }
-  else
-    return alist;
+    alist = scm_delq1_x (handle, alist);
+
+  return alist;
 }
 #undef FUNC_NAME
 
@@ -355,11 +373,9 @@ SCM_DEFINE (scm_assv_remove_x, "assv-remove!", 2, 0, 0,
 
   handle = scm_sloppy_assv (key, alist);
   if (SCM_CONSP (handle))
-    {
-      return scm_delv_x (handle, alist);
-    }
-  else
-    return alist;
+    alist = scm_delq1_x (handle, alist);
+
+  return alist;
 }
 #undef FUNC_NAME
 
@@ -373,11 +389,9 @@ SCM_DEFINE (scm_assoc_remove_x, "assoc-remove!", 2, 0, 0,
 
   handle = scm_sloppy_assoc (key, alist);
   if (SCM_CONSP (handle))
-    {
-      return scm_delete_x (handle, alist);
-    }
-  else
-    return alist;
+    alist = scm_delq1_x (handle, alist);
+
+  return alist;
 }
 #undef FUNC_NAME
 
