@@ -3,7 +3,7 @@
 #ifndef COOP_DEFSH
 #define COOP_DEFSH
 
-/*	Copyright (C) 1996, 1997, 1998, 1999, 2000, 2002 Free Software Foundation, Inc.
+/* Copyright (C) 1996,1997,1998,1999,2000,2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -141,7 +141,7 @@ typedef struct coop_m {
 
 typedef int coop_mattr;
 
-typedef coop_m scm_mutex_t;
+typedef coop_m scm_t_mutex;
 
 extern int coop_mutex_init (coop_m*);
 extern int coop_new_mutex_init (coop_m*, coop_mattr*);
@@ -164,7 +164,7 @@ typedef struct coop_c {
 
 typedef int coop_cattr;
 
-typedef coop_c scm_cond_t;
+typedef coop_c scm_t_cond;
 
 #ifndef HAVE_STRUCT_TIMESPEC
 /* POSIX.4 structure for a time value.  This is like a `struct timeval' but
@@ -193,9 +193,9 @@ extern int coop_condition_variable_destroy (coop_c*);
 
 typedef int coop_k;
 
-typedef coop_k scm_key_t;
+typedef coop_k scm_t_key;
 
-extern int coop_key_create (coop_k *keyp, void (*destructor) (void *value));
+extern int coop_key_create (coop_k *keyp, void (*destruktor) (void *value));
 extern int coop_setspecific (coop_k key, const void *value);
 extern void *coop_getspecific (coop_k key);
 extern int coop_key_delete (coop_k);
@@ -233,8 +233,8 @@ extern coop_t *coop_wait_for_runnable_thread (void);
 
 /* Cooperative threads don't need to have these defined */
 
-#define SCM_THREAD_CRITICAL_SECTION_START 
-#define SCM_THREAD_CRITICAL_SECTION_END 
+#define SCM_CRITICAL_SECTION_START 
+#define SCM_CRITICAL_SECTION_END 
 
 
 
@@ -243,12 +243,6 @@ extern coop_t *coop_wait_for_runnable_thread (void);
 
 
 
-#define SCM_THREAD_DEFER
-#define SCM_THREAD_ALLOW
-#define SCM_THREAD_REDEFER
-#define SCM_THREAD_REALLOW_1
-#define SCM_THREAD_REALLOW_2
-
 #if 0
 #define SCM_THREAD_SWITCHING_CODE \
 do { \
