@@ -1,4 +1,4 @@
-/*      Copyright (C) 1995,1996,1997, 2000 Free Software Foundation, Inc.
+/*      Copyright (C) 1995,1996,1997, 2000, 2001 Free Software Foundation, Inc.
 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,8 +42,6 @@
 
 /* list manipulation */
 
-#include <stdio.h>
-
 #include "libguile/gh.h"
 
 /* returns the length of a list */
@@ -60,24 +58,28 @@ gh_length (SCM l)
 /* gh_append() takes a args, which is a list of lists, and appends
    them all together into a single list, which is returned.  This is
    equivalent to the Scheme procedure (append list1 list2 ...) */
-SCM gh_append(SCM args)
+SCM
+gh_append (SCM args)
 {
-  return scm_append(args);
+  return scm_append (args);
 }
 
-SCM gh_append2(SCM l1, SCM l2)
+SCM
+gh_append2 (SCM l1, SCM l2)
 {
-  return scm_append(scm_listify(l1, l2, SCM_UNDEFINED));
+  return scm_append (scm_list_2 (l1, l2));
 }
 
-SCM gh_append3(SCM l1, SCM l2, SCM l3)
+SCM
+gh_append3(SCM l1, SCM l2, SCM l3)
 {
-  return scm_append(scm_listify(l1, l2, l3, SCM_UNDEFINED));
+  return scm_append (scm_list_3 (l1, l2, l3));
 }
 
-SCM gh_append4(SCM l1, SCM l2, SCM l3, SCM l4)
+SCM
+gh_append4 (SCM l1, SCM l2, SCM l3, SCM l4)
 {
-  return scm_append(scm_listify(l1, l2, l3, l4, SCM_UNDEFINED));
+  return scm_append (scm_list_4 (l1, l2, l3, l4));
 }
 
 /* gh_reverse() is defined as a macro in gh.h */
