@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995-1999, 2000, 2002 Free Software Foundation, Inc.
+/* Copyright (C) 1995-1999,2000,2001, 2003 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,9 +42,11 @@
 
 
 
-#include <stdio.h>
+#include <errno.h>
+
 #include "libguile/_scm.h"
 #include "libguile/chars.h"
+#include "libguile/continuations.h"
 #include "libguile/smob.h"
 #include "libguile/eval.h"
 #include "libguile/macros.h"
@@ -119,20 +121,24 @@ char *scm_isymnames[] =
   "#@bind",
   
   "#@delay",
-  
+  "#@call-with-values",
+
   "#<unbound>"
 };
 
-scm_option scm_print_opts[] = {
-  { SCM_OPTION_SCM, "closure-hook", SCM_UNPACK(SCM_BOOL_F),
-    "Hook for printing closures." },
+scm_t_option scm_print_opts[] = {
+  { SCM_OPTION_SCM, "closure-hook", SCM_UNPACK (SCM_BOOL_F),
+    "Hook for printing closures (should handle macros as well)." },
   { SCM_OPTION_BOOLEAN, "source", 0,
     "Print closures with source." }
 };
 
 SCM_DEFINE (scm_print_options, "print-options-interface", 0, 1, 0, 
             (SCM setting),
-"")
+	    "Option interface for the print options. Instead of using\n"
+	    "this procedure directly, use the procedures\n"
+	    "@code{print-enable}, @code{print-disable}, @code{print-set!}\n"
+	    "and @code{print-options}.")
 #define FUNC_NAME s_scm_print_options
 {
   SCM ans = scm_options (setting,
@@ -187,8 +193,9 @@ static SCM print_state_pool;
 
 SCM_DEFINE (scm_current_pstate, "current-pstate", 0, 0, 0, 
            (),
-            "Return the current-pstate--the `cadr' of the print_state_pool.\n"
-            "`current-pstate' is only included in GUILE_DEBUG builds.")
+	    "Return the current-pstate -- the cadr of the\n"
+	    "@code{print_state_pool}.  @code{current-pstate} is only\n"
+	    "included in @code{--enable-guile-debug} builds.")
 #define FUNC_NAME s_scm_current_pstate
 {
   if (SCM_NNULLP (SCM_CDR (print_state_pool)))
@@ -209,10 +216,9 @@ make_print_state (void)
 				     SCM_INUM0,
 				     SCM_EOL);
   scm_print_state *pstate = SCM_PRINT_STATE (print_state);
-  pstate->ref_vect = scm_make_vector (SCM_MAKINUM (PSTATE_SIZE),
-				      SCM_UNDEFINED);
+  pstate->ref_vect = scm_c_make_vector (PSTATE_SIZE, SCM_UNDEFINED);
   pstate->ref_stack = SCM_VELTS (pstate->ref_vect);
-  pstate->ceiling = SCM_LENGTH (pstate->ref_vect);
+  pstate->ceiling = SCM_VECTOR_LENGTH (pstate->ref_vect);
   return print_state;
 }
 
@@ -247,8 +253,8 @@ scm_free_print_state (SCM print_state)
   pstate->revealed = 0;
   SCM_NEWCELL (handle);
   SCM_DEFER_INTS;
-  SCM_SETCAR (handle, print_state);
-  SCM_SETCDR (handle, SCM_CDR (print_state_pool));
+  SCM_SET_CELL_WORD_0 (handle, print_state);
+  SCM_SET_CELL_WORD_1 (handle, SCM_CDR (print_state_pool));
   SCM_SETCDR (print_state_pool, handle);
   SCM_ALLOW_INTS;
 }
@@ -256,9 +262,18 @@ scm_free_print_state (SCM print_state)
 static void
 grow_ref_stack (scm_print_state *pstate)
 {
-  int new_size = 2 * pstate->ceiling;
-  scm_vector_set_length_x (pstate->ref_vect, SCM_MAKINUM (new_size));
-  pstate->ref_stack = SCM_VELTS (pstate->ref_vect);
+  unsigned long int old_size = SCM_VECTOR_LENGTH (pstate->ref_vect);
+  SCM *old_elts = SCM_VELTS (pstate->ref_vect);
+  unsigned long int new_size = 2 * pstate->ceiling;
+  SCM new_vect = scm_c_make_vector (new_size, SCM_UNDEFINED);
+  SCM *new_elts = SCM_VELTS (new_vect);
+  unsigned long int i;
+
+  for (i = 0; i != old_size; ++i)
+    new_elts [i] = old_elts [i];
+
+  pstate->ref_vect = new_vect;
+  pstate->ref_stack = new_elts;
   pstate->ceiling = new_size;
 }
 
@@ -266,8 +281,8 @@ grow_ref_stack (scm_print_state *pstate)
 static void
 print_circref (SCM port,scm_print_state *pstate,SCM ref)
 {
-  register int i;
-  int self = pstate->top - 1;
+  register long i;
+  long self = pstate->top - 1;
   i = pstate->top - 1;
   if (SCM_CONSP (pstate->ref_stack[i]))
     {
@@ -289,6 +304,91 @@ print_circref (SCM port,scm_print_state *pstate,SCM ref)
   scm_putc ('#', port);
 }
 
+/* Print the name of a symbol. */
+
+void
+scm_print_symbol_name (const char *str, size_t len, SCM port)
+{
+  size_t pos;
+  size_t end;
+  int weird;
+  int maybe_weird;
+  size_t mw_pos = 0;
+  
+  pos = 0;
+  weird = 0;
+  maybe_weird = 0;
+  
+  /* XXX - Lots of weird symbol names are missed, such as "12" or
+     "'a". */
+
+  if (len == 0)
+    scm_lfwrite ("#{}#", 4, port);
+  else if (str[0] == '#' || str[0] == ':' || str[len-1] == ':')
+    {
+      scm_lfwrite ("#{", 2, port);
+      weird = 1;
+    }
+  
+  for (end = pos; end < len; ++end)
+    switch (str[end])
+      {
+#ifdef BRACKETS_AS_PARENS
+      case '[':
+      case ']':
+#endif
+      case '(':
+      case ')':
+      case '"':
+      case ';':
+      case SCM_WHITE_SPACES:
+      case SCM_LINE_INCREMENTORS:
+      weird_handler:
+	if (maybe_weird)
+	  {
+	    end = mw_pos;
+	    maybe_weird = 0;
+	  }
+	if (!weird)
+	  {
+	    scm_lfwrite ("#{", 2, port);
+	    weird = 1;
+	  }
+	if (pos < end)
+	  {
+	    scm_lfwrite (str + pos, end - pos, port);
+	  }
+	{
+	  char buf[2];
+	  buf[0] = '\\';
+	  buf[1] = str[end];
+	  scm_lfwrite (buf, 2, port);
+	}
+	pos = end + 1;
+	break;
+      case '\\':
+	if (weird)
+	  goto weird_handler;
+	if (!maybe_weird)
+	  {
+	    maybe_weird = 1;
+	    mw_pos = pos;
+	  }
+	break;
+      case '}':
+      case '#':
+	if (weird)
+	  goto weird_handler;
+	break;
+      default:
+	break;
+      }
+  if (pos < end)
+    scm_lfwrite (str + pos, end - pos, port);
+  if (weird)
+    scm_lfwrite ("}#", 2, port);
+}
+
 /* Print generally.  Handles both write and display according to PSTATE.
  */
 SCM_GPROC(s_write, "write", 1, 1, 0, scm_write, g_write);
@@ -300,21 +400,33 @@ scm_iprin1 (SCM exp, SCM port, scm_print_state *pstate)
 taloop:
   switch (SCM_ITAG3 (exp))
     {
-    case 2:
-    case 6:
+    case scm_tc3_closure:
+    case scm_tc3_tc7_1:
+    case scm_tc3_tc7_2:
+      /* These tc3 tags should never occur in an immediate value.  They are
+       * only used in cell types of non-immediates, i. e. the value returned
+       * by SCM_CELL_TYPE (exp) can use these tags.
+       */
+      scm_ipruk ("immediate", exp, port);
+      break;
+    case scm_tc3_int_1:
+    case scm_tc3_int_2:
       scm_intprint (SCM_INUM (exp), 10, port);
       break;
-    case 4:
+    case scm_tc3_imm24:
       if (SCM_CHARP (exp))
 	{
-	  register long i;
+	  long i = SCM_CHAR (exp);
 
-	  i = SCM_CHAR (exp);
 	  if (SCM_WRITINGP (pstate))
 	    {
 	      scm_puts ("#\\", port);
 	      if ((i >= 0) && (i <= ' ') && scm_charnames[i])
 		scm_puts (scm_charnames[i], port);
+#ifndef EBCDIC
+	      else if (i == '\177')
+		scm_puts (scm_charnames[scm_n_charnames - 1], port);
+#endif
 	      else if (i < 0 || i > '\177')
 		scm_intprint (i, 8, port);
 	      else
@@ -329,23 +441,23 @@ taloop:
       else if (SCM_ILOCP (exp))
 	{
 	  scm_puts ("#@", port);
-	  scm_intprint (SCM_IFRAME (exp), 10, port);
+	  scm_intprint ((long) SCM_IFRAME (exp), 10, port);
 	  scm_putc (SCM_ICDRP (exp) ? '-' : '+', port);
-	  scm_intprint (SCM_IDIST (exp), 10, port);
+	  scm_intprint ((long) SCM_IDIST (exp), 10, port);
 	}
       else
-	goto idef;
+	{
+	  /* unknown immediate value */
+	  scm_ipruk ("immediate", exp, port);
+	}
       break;
-    case 1:
+    case scm_tc3_cons_gloc:
       /* gloc */
       scm_puts ("#@", port);
-      exp = SCM_GLOC_SYM (exp);
+      exp = scm_module_reverse_lookup (scm_current_module (),
+				       SCM_GLOC_VAR (exp));
       goto taloop;
-    default:
-    idef:
-      scm_ipruk ("immediate", exp, port);
-      break;
-    case 0:
+    case scm_tc3_cons:
       switch (SCM_TYP7 (exp))
 	{
 	case scm_tcs_cons_gloc:
@@ -361,6 +473,7 @@ taloop:
 		  SCM_NEWSMOB (pwps,
 			       scm_tc16_port_with_ps,
 			       SCM_UNPACK (scm_cons (port, pstate->handle)));
+		  pstate->revealed = 1;
 		  scm_call_generic_2 (print, exp, pwps);
 		}
 	      else
@@ -381,181 +494,58 @@ taloop:
 	circref:
 	  print_circref (port, pstate, exp);
 	  break;
-	macros:
-	  if (!SCM_CLOSUREP (SCM_CDR (exp)))
-	    goto prinmacro;
 	case scm_tcs_closures:
-	  /* The user supplied print closure procedure must handle
-	     macro closures as well. */
 	  if (SCM_FALSEP (scm_procedure_p (SCM_PRINT_CLOSURE))
 	      || SCM_FALSEP (scm_printer_apply (SCM_PRINT_CLOSURE,
 						exp, port, pstate)))
-	  {
-	    SCM name, code, env;
-	    if (SCM_TYP16 (exp) == scm_tc16_macro)
-	      {
-		/* Printing a macro. */
-	      prinmacro:
-		name = scm_macro_name (exp);
-		if (!SCM_CLOSUREP (SCM_CDR (exp)))
-		  {
-		    code = env = SCM_UNDEFINED;
-		    scm_puts ("#<primitive-", port);
-		  }
-		else
-		  {
-		    code = SCM_CODE (SCM_CDR (exp));
-		    env = SCM_ENV (SCM_CDR (exp));
-		    scm_puts ("#<", port);
-		  }
-		if (SCM_CELL_WORD_0 (exp) & (3L << 16))
-		  scm_puts ("macro", port);
-		else
-		  scm_puts ("syntax", port);
-		if (SCM_CELL_WORD_0 (exp) & (2L << 16))
-		  scm_putc ('!', port);
-	      }
-	    else
-	      {
-		/* Printing a closure. */
-		name = scm_procedure_name (exp);
-		code = SCM_CODE (exp);
-		env = SCM_ENV (exp);
-		scm_puts ("#<procedure", port);
-	      }
-	    if (SCM_ROSTRINGP (name))
-	      {
-		scm_putc (' ', port);
-		scm_puts (SCM_ROCHARS (name), port);
-	      }
-	    if (!SCM_UNBNDP (code))
-	      {
-		if (SCM_PRINT_SOURCE_P)
-		  {
-		    code = scm_unmemocopy (code,
-					   SCM_EXTEND_ENV (SCM_CAR (code),
-							   SCM_EOL,
-							   env));
-		    ENTER_NESTED_DATA (pstate, exp, circref);
-		    scm_iprlist (" ", code, '>', port, pstate);
-		    EXIT_NESTED_DATA (pstate);
-		  }
-		else
-		  {
-		    if (SCM_TYP16 (exp) != scm_tc16_macro)
-		      {
-			scm_putc (' ', port);
-			scm_iprin1 (SCM_CAR (code), port, pstate);
-		      }
-		    scm_putc ('>', port);
-		  }
-	      }
-	    else
+	    {
+	      SCM formals = SCM_CLOSURE_FORMALS (exp);
+	      scm_puts ("#<procedure", port);
+	      scm_putc (' ', port);
+	      scm_iprin1 (scm_procedure_name (exp), port, pstate);
+	      scm_putc (' ', port);
+	      if (SCM_PRINT_SOURCE_P)
+		{
+		  SCM env = SCM_ENV (exp);
+		  SCM xenv = SCM_EXTEND_ENV (formals, SCM_EOL, env);
+		  SCM src = scm_unmemocopy (SCM_CODE (exp), xenv);
+		  ENTER_NESTED_DATA (pstate, exp, circref);
+		  scm_iprin1 (src, port, pstate);
+		  EXIT_NESTED_DATA (pstate);
+		}
+	      else
+		scm_iprin1 (formals, port, pstate);
 	      scm_putc ('>', port);
-	  }
+	    }
 	  break;
 	case scm_tc7_substring:
 	case scm_tc7_string:
 	  if (SCM_WRITINGP (pstate))
 	    {
-	      scm_sizet i;
+	      size_t i;
 
 	      scm_putc ('"', port);
-	      for (i = 0; i < SCM_ROLENGTH (exp); ++i)
-		switch (SCM_ROCHARS (exp)[i])
+	      for (i = 0; i < SCM_STRING_LENGTH (exp); ++i)
+		switch (SCM_STRING_CHARS (exp)[i])
 		  {
 		  case '"':
 		  case '\\':
 		    scm_putc ('\\', port);
 		  default:
-		    scm_putc (SCM_ROCHARS (exp)[i], port);
+		    scm_putc (SCM_STRING_CHARS (exp)[i], port);
 		  }
 	      scm_putc ('"', port);
 	      break;
 	    }
 	  else
-	    scm_lfwrite (SCM_ROCHARS (exp), (scm_sizet) SCM_ROLENGTH (exp),
-			 port);
+	    scm_lfwrite (SCM_STRING_CHARS (exp), SCM_STRING_LENGTH (exp), port);
+	  break;
+	case scm_tc7_symbol:
+	  scm_print_symbol_name (SCM_SYMBOL_CHARS (exp),
+				 SCM_SYMBOL_LENGTH (exp),
+				 port);
+	  scm_remember_upto_here_1 (exp);
 	  break;
-	case scm_tcs_symbols:
-	    {
-	      int pos;
-	      int end;
-	      int len;
-	      char * str;
-	      int weird;
-	      int maybe_weird;
-	      int mw_pos = 0;
-
-	      len = SCM_LENGTH (exp);
-	      str = SCM_CHARS (exp);
-	      scm_remember (&exp);
-	      pos = 0;
-	      weird = 0;
-	      maybe_weird = 0;
-
-	      if (len == 0)
-		scm_lfwrite ("#{}#", 4, port);
-
-	      for (end = pos; end < len; ++end)
-		switch (str[end])
-		  {
-#ifdef BRACKETS_AS_PARENS
-		  case '[':
-		  case ']':
-#endif
-		  case '(':
-		  case ')':
-		  case '"':
-		  case ';':
-		  case SCM_WHITE_SPACES:
-		  case SCM_LINE_INCREMENTORS:
-		  weird_handler:
-		    if (maybe_weird)
-		      {
-			end = mw_pos;
-			maybe_weird = 0;
-		      }
-		    if (!weird)
-		      {
-			scm_lfwrite ("#{", 2, port);
-			weird = 1;
-		      }
-		    if (pos < end)
-		      {
-			scm_lfwrite (str + pos, end - pos, port);
-		      }
-		    {
-		      char buf[2];
-		      buf[0] = '\\';
-		      buf[1] = str[end];
-		      scm_lfwrite (buf, 2, port);
-		    }
-		    pos = end + 1;
-		    break;
-		  case '\\':
-		    if (weird)
-		      goto weird_handler;
-		    if (!maybe_weird)
-		      {
-			maybe_weird = 1;
-			mw_pos = pos;
-		      }
-		    break;
-		  case '}':
-		  case '#':
-		    if (weird)
-		      goto weird_handler;
-		    break;
-		  default:
-		    break;
-		  }
-	      if (pos < end)
-		scm_lfwrite (str + pos, end - pos, port);
-	      if (weird)
-		scm_lfwrite ("}#", 2, port);
-	      break;
-	    }
 	case scm_tc7_wvect:
 	  ENTER_NESTED_DATA (pstate, exp, circref);
 	  if (SCM_IS_WHVEC (exp))
@@ -570,9 +560,9 @@ taloop:
 	common_vector_printer:
 	  {
 	    register long i;
-	    int last = SCM_LENGTH (exp) - 1;
+	    long last = SCM_VECTOR_LENGTH (exp) - 1;
 	    int cutp = 0;
-	    if (pstate->fancyp && SCM_LENGTH (exp) > pstate->length)
+	    if (pstate->fancyp && SCM_VECTOR_LENGTH (exp) > pstate->length)
 	      {
 		last = pstate->length - 1;
 		cutp = 1;
@@ -614,7 +604,7 @@ taloop:
 		    ? "#<primitive-generic "
 		    : "#<primitive-procedure ",
 		    port);
-	  scm_puts (SCM_CHARS (SCM_SNAME (exp)), port);
+	  scm_puts (SCM_SYMBOL_CHARS (SCM_SNAME (exp)), port);
 	  scm_putc ('>', port);
 	  break;
 #ifdef CCLO
@@ -629,7 +619,7 @@ taloop:
 		if (SCM_NFALSEP (name))
 		  {
 		    scm_putc (' ', port);
-		    scm_puts (SCM_CHARS (name), port);
+		    scm_puts (SCM_SYMBOL_CHARS (name), port);
 		  }
 	      }
 	    else
@@ -648,18 +638,11 @@ taloop:
 	    if (SCM_NFALSEP (name))
 	      {
 		scm_putc (' ', port);
-		scm_puts (SCM_ROCHARS (name), port);
+		scm_display (name, port);
 	      }
 	  }
 	  scm_putc ('>', port);
 	  break;
-	case scm_tc7_contin:
-	  scm_puts ("#<continuation ", port);
-	  scm_intprint (SCM_LENGTH (exp), 10, port);
-	  scm_puts (" @ ", port);
-	  scm_intprint ((long) SCM_CHARS (exp), 16, port);
-	  scm_putc ('>', port);
-	  break;
 	case scm_tc7_port:
 	  {
 	    register long i = SCM_PTOBNUM (exp);
@@ -670,24 +653,10 @@ taloop:
 	    goto punk;
 	  }
 	case scm_tc7_smob:
-	  {
-	    register long i;
-	    ENTER_NESTED_DATA (pstate, exp, circref);
-	    i = SCM_SMOBNUM (exp);
-	    if (i < scm_numsmob && scm_smobs[i].print
-		&& (scm_smobs[i].print) (exp, port, pstate))
-	      {
-		EXIT_NESTED_DATA (pstate);
-		break;
-	      }
-	    EXIT_NESTED_DATA (pstate);
-	    /* Macros have their print field set to NULL.  They are
-	       handled at the same place as closures in order to achieve
-	       non-redundancy.  Placing the condition here won't slow
-	       down printing of other smobs. */
-	    if (SCM_TYP16 (exp) == scm_tc16_macro)
-	      goto macros;
-	  }
+	  ENTER_NESTED_DATA (pstate, exp, circref);
+	  SCM_SMOB_DESCRIPTOR (exp).print (exp, port, pstate);
+	  EXIT_NESTED_DATA (pstate);
+	  break;
 	default:
 	punk:
 	  scm_ipruk ("type", exp, port);
@@ -710,6 +679,7 @@ scm_prin1 (SCM exp, SCM port, int writingp)
   SCM handle = SCM_BOOL_F; /* Will GC protect the handle whilst unlinked */
   SCM pstate_scm;
   scm_print_state *pstate;
+  int old_writingp;
 
   /* If PORT is a print-state/port pair, use that.  Else create a new
      print-state. */
@@ -735,8 +705,10 @@ scm_prin1 (SCM exp, SCM port, int writingp)
     }
 
   pstate = SCM_PRINT_STATE (pstate_scm);
+  old_writingp = pstate->writingp;
   pstate->writingp = writingp;
   scm_iprin1 (exp, port, pstate);
+  pstate->writingp = old_writingp;
 
   /* Return print state to pool if it has been created above and
      hasn't escaped to Scheme. */
@@ -782,15 +754,16 @@ scm_ipruk (char *hdr, SCM ptr, SCM port)
   scm_putc ('>', port);
 }
 
-/* Print a list.
+
+/* Print a list.  The list may be either a list of ordinary data, or it may be
+   a list that represents code.  Lists that represent code may contain gloc
+   cells.
  */
-
-
 void 
 scm_iprlist (char *hdr,SCM exp,int tlr,SCM port,scm_print_state *pstate)
 {
   register SCM hare, tortoise;
-  int floor = pstate->top - 2;
+  long floor = pstate->top - 2;
   scm_puts (hdr, port);
   /* CHECK_INTS; */
   if (pstate->fancyp)
@@ -813,13 +786,10 @@ scm_iprlist (char *hdr,SCM exp,int tlr,SCM port,scm_print_state *pstate)
   
   /* No cdr cycles intrinsic to this list */
   scm_iprin1 (SCM_CAR (exp), port, pstate);
-  exp = SCM_CDR (exp);
-  for (; SCM_NIMP (exp); exp = SCM_CDR (exp))
+  for (exp = SCM_CDR (exp); SCM_ECONSP (exp); exp = SCM_CDR (exp))
     {
-      register int i;
+      register long i;
 
-      if (SCM_NECONSP (exp))
-	break;
       for (i = floor; i >= 0; --i)
 	if (SCM_EQ_P (pstate->ref_stack[i], exp))
 	  goto circref;
@@ -828,7 +798,7 @@ scm_iprlist (char *hdr,SCM exp,int tlr,SCM port,scm_print_state *pstate)
       /* CHECK_INTS; */
       scm_iprin1 (SCM_CAR (exp), port, pstate);
     }
-  if (SCM_NNULLP (exp))
+  if (!SCM_NULLP (exp))
     {
       scm_puts (" . ", port);
       scm_iprin1 (exp, port, pstate);
@@ -841,16 +811,14 @@ end:
   
 fancy_printing:
   {
-    int n = pstate->length;
+    long n = pstate->length;
     
     scm_iprin1 (SCM_CAR (exp), port, pstate);
     exp = SCM_CDR (exp); --n;
-    for (; SCM_NIMP (exp); exp = SCM_CDR (exp))
+    for (; SCM_ECONSP (exp); exp = SCM_CDR (exp))
       {
 	register unsigned long i;
 
-	if (SCM_NECONSP (exp))
-	  break;
 	for (i = 0; i < pstate->top; ++i)
 	  if (SCM_EQ_P (pstate->ref_stack[i], exp))
 	    goto fancy_circref;
@@ -943,68 +911,105 @@ scm_display (SCM obj, SCM port)
 
 SCM_DEFINE (scm_simple_format, "simple-format", 2, 0, 1,
             (SCM destination, SCM message, SCM args),
-	    "Write MESSAGE to DESTINATION, defaulting to `current-output-port'.\n"
-	    "MESSAGE can contain ~A (was %s) and ~S (was %S) escapes.  When printed,\n"
-	    "the escapes are replaced with corresponding members of ARGS:\n"
-	    "~A formats using `display' and ~S formats using `write'.\n"
-	    "If DESTINATION is #t, then use the `current-output-port',\n"
-	    "if DESTINATION is #f, then return a string containing the formatted text.\n"
-	    "Does not add a trailing newline.")
+	    "Write @var{message} to @var{destination}, defaulting to\n"
+	    "the current output port.\n"
+	    "@var{message} can contain @code{~A} (was @code{%s}) and\n"
+	    "@code{~S} (was @code{%S}) escapes.  When printed,\n"
+	    "the escapes are replaced with corresponding members of\n"
+	    "@var{ARGS}:\n"
+	    "@code{~A} formats using @code{display} and @code{~S} formats\n"
+	    "using @code{write}.\n"
+	    "If @var{destination} is @code{#t}, then use the current output\n"
+	    "port, if @var{destination} is @code{#f}, then return a string\n"
+	    "containing the formatted text. Does not add a trailing newline.")
 #define FUNC_NAME s_scm_simple_format
 {
   SCM answer = SCM_UNSPECIFIED;
   int fReturnString = 0;
   int writingp;
   char *start;
+  char *end;
   char *p;
 
-  if (SCM_EQ_P (destination, SCM_BOOL_T)) {
-    destination = scm_cur_outp;
-  } else if (SCM_FALSEP (destination)) {
-    fReturnString = 1;
-    destination = scm_mkstrport (SCM_INUM0, 
-                          scm_make_string (SCM_INUM0, SCM_UNDEFINED),
-                          SCM_OPN | SCM_WRTNG,
-                          FUNC_NAME);
-  } else {
-    SCM_VALIDATE_OPORT_VALUE (1,destination);
-  }
-  SCM_VALIDATE_STRING(2,message);
+  if (SCM_EQ_P (destination, SCM_BOOL_T))
+    {
+      destination = scm_cur_outp;
+    }
+  else if (SCM_FALSEP (destination))
+    {
+      fReturnString = 1;
+      destination = scm_mkstrport (SCM_INUM0, 
+				   scm_make_string (SCM_INUM0, SCM_UNDEFINED),
+				   SCM_OPN | SCM_WRTNG,
+				   FUNC_NAME);
+    }
+  else
+    {
+      SCM_VALIDATE_OPORT_VALUE (1, destination);
+      destination = SCM_COERCE_OUTPORT (destination);
+    }
+  SCM_VALIDATE_STRING (2, message);
   SCM_VALIDATE_REST_ARGUMENT (args);
 
-  start = SCM_ROCHARS (message);
-  for (p = start; *p != '\0'; ++p)
+  start = SCM_STRING_CHARS (message);
+  end = start + SCM_STRING_LENGTH (message);
+  for (p = start; p != end; ++p)
     if (*p == '~')
       {
-	if (SCM_IMP (args) || SCM_NCONSP (args))
-	  continue;
-	
-	++p;
-	if (*p == 'A')
-	  writingp = 0;
-	else if (*p == 'S')
-	  writingp = 1;
-	else
-	  continue;
+	if (++p == end)
+	  break;
 
+	switch (*p) 
+	  {
+	  case 'A': case 'a':
+	    writingp = 0;
+	    break;
+	  case 'S': case 's':
+	    writingp = 1;
+	    break;
+	  case '~':
+	    scm_lfwrite (start, p - start, destination);
+	    start = p + 1;
+	    continue;
+	  case '%':
+	    scm_lfwrite (start, p - start - 1, destination);
+	    scm_newline (destination);
+	    start = p + 1;
+	    continue;
+	  default:
+	    SCM_MISC_ERROR ("FORMAT: Unsupported format option ~~~A - use (ice-9 format) instead",
+			    scm_list_1 (SCM_MAKE_CHAR (*p)));
+	    
+	  }
+
+
+	if (!SCM_CONSP (args))
+	  SCM_MISC_ERROR ("FORMAT: Missing argument for ~~~A",
+			  scm_list_1 (SCM_MAKE_CHAR (*p)));
+			  		
 	scm_lfwrite (start, p - start - 1, destination);
 	scm_prin1 (SCM_CAR (args), destination, writingp);
 	args = SCM_CDR (args);
 	start = p + 1;
       }
+
   scm_lfwrite (start, p - start, destination);
+  if (args != SCM_EOL)
+    SCM_MISC_ERROR ("FORMAT: ~A superfluous arguments",
+		    scm_list_1 (scm_length (args)));
 
   if (fReturnString)
     answer = scm_strport_to_string (destination);
 
-  return scm_return_first(answer,message);
+  return scm_return_first (answer, message);
 }
 #undef FUNC_NAME
 
 
 SCM_DEFINE (scm_newline, "newline", 0, 1, 0, 
             (SCM port),
-	    "Send a newline to PORT.")
+	    "Send a newline to @var{port}.\n"
+	    "If @var{port} is omitted, send to the current output port.")
 #define FUNC_NAME s_scm_newline
 {
   if (SCM_UNBNDP (port))
@@ -1019,7 +1024,7 @@ SCM_DEFINE (scm_newline, "newline", 0, 1, 0,
 
 SCM_DEFINE (scm_write_char, "write-char", 1, 1, 0,
             (SCM chr, SCM port),
-	    "Send character CHR to PORT.")
+	    "Send character @var{chr} to @var{port}.")
 #define FUNC_NAME s_scm_write_char
 {
   if (SCM_UNBNDP (port))
@@ -1049,12 +1054,12 @@ SCM_DEFINE (scm_write_char, "write-char", 1, 1, 0,
  * escaped to Scheme and thus has to be freed by the GC.
  */
 
-long scm_tc16_port_with_ps;
+scm_t_bits scm_tc16_port_with_ps;
 
 /* Print exactly as the port itself would */
 
 static int
-print_port_with_ps (SCM obj, SCM port, scm_print_state *pstate)
+port_with_ps_print (SCM obj, SCM port, scm_print_state *pstate)
 {
   obj = SCM_PORT_WITH_PS_PORT (obj);
   return scm_ptobs[SCM_PTOBNUM (obj)].print (obj, port, pstate);
@@ -1067,12 +1072,13 @@ scm_printer_apply (SCM proc, SCM exp, SCM port, scm_print_state *pstate)
   SCM pair = scm_cons (port, pstate->handle);
   SCM_NEWSMOB (pwps, scm_tc16_port_with_ps, SCM_UNPACK (pair));
   pstate->revealed = 1;
-  return scm_apply (proc, exp, scm_cons (pwps, scm_listofnull));
+  return scm_call_2 (proc, exp, pwps);
 }
 
 SCM_DEFINE (scm_port_with_print_state, "port-with-print-state", 2, 0, 0, 
             (SCM port, SCM pstate),
-	    "")
+	    "Create a new port which behaves like @var{port}, but with an\n"
+	    "included print state @var{pstate}.")
 #define FUNC_NAME s_scm_port_with_print_state
 {
   SCM pwps;
@@ -1086,14 +1092,15 @@ SCM_DEFINE (scm_port_with_print_state, "port-with-print-state", 2, 0, 0,
 
 SCM_DEFINE (scm_get_print_state, "get-print-state", 1, 0, 0, 
             (SCM port),
-	    "")
+	    "Return the print state of the port @var{port}. If @var{port}\n"
+	    "has no associated print state, @code{#f} is returned.")
 #define FUNC_NAME s_scm_get_print_state
 {
   if (SCM_PORT_WITH_PS_P (port))
     return SCM_PORT_WITH_PS_PS (port);
   if (SCM_OUTPUT_PORT_P (port))
     return SCM_BOOL_F;
-  RETURN_SCM_WTA (1,port);
+  SCM_WRONG_TYPE_ARG (1, port);
 }
 #undef FUNC_NAME
 
@@ -1105,12 +1112,10 @@ scm_init_print ()
   SCM vtable, layout, type;
   
   scm_init_opts (scm_print_options, scm_print_opts, SCM_N_PRINT_OPTIONS);
-  vtable = scm_make_vtable_vtable (scm_make_struct_layout (scm_nullstr),
-				   SCM_INUM0,
-				   SCM_EOL);
+  vtable = scm_make_vtable_vtable (scm_nullstr, SCM_INUM0, SCM_EOL);
   layout = scm_make_struct_layout (scm_makfrom0str (SCM_PRINT_STATE_LAYOUT));
-  type = scm_make_struct (vtable, SCM_INUM0, SCM_LIST1 (layout));
-  scm_set_struct_vtable_name_x (type, SCM_CAR (scm_intern0 ("print-state")));
+  type = scm_make_struct (vtable, SCM_INUM0, scm_list_1 (layout));
+  scm_set_struct_vtable_name_x (type, scm_str2symbol ("print-state"));
   print_state_pool = scm_permanent_object (scm_cons (type, SCM_EOL));
 
   scm_print_state_vtable = type;
@@ -1118,7 +1123,7 @@ scm_init_print ()
   /* Don't want to bind a wrapper class in GOOPS, so pass 0 as arg1. */
   scm_tc16_port_with_ps = scm_make_smob_type (0, 0);
   scm_set_smob_mark (scm_tc16_port_with_ps, scm_markcdr);
-  scm_set_smob_print (scm_tc16_port_with_ps, print_port_with_ps);
+  scm_set_smob_print (scm_tc16_port_with_ps, port_with_ps_print);
   
 #include "libguile/print.x"
 }
