@@ -1,8 +1,8 @@
 /* classes: h_files */
 
-#ifndef PRINTH
-#define PRINTH
-/*	Copyright (C) 1995,1996,1998, 2000, 2002 Free Software Foundation, Inc.
+#ifndef SCM_PRINT_H
+#define SCM_PRINT_H
+/* Copyright (C) 1995,1996,1998,2000,2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@
 
 #include "libguile/options.h"
 
-extern scm_option scm_print_opts[];
+extern scm_t_option scm_print_opts[];
 
 #define SCM_PRINT_CLOSURE	(SCM_PACK (scm_print_opts[0].val))
 #define SCM_PRINT_SOURCE_P	((int) scm_print_opts[1].val)
@@ -71,13 +71,12 @@ do { \
 #define SCM_WRITINGP(pstate) ((pstate)->writingp)
 #define SCM_SET_WRITINGP(pstate, x) { (pstate)->writingp = (x); }
 
-#define SCM_PORT_WITH_PS_P(p) (SCM_NIMP(p) && (SCM_TYP16 (p) == scm_tc16_port_with_ps))
-#define SCM_PORT_WITH_PS_PORT(p) SCM_CADR (p)
-#define SCM_PORT_WITH_PS_PS(p) SCM_CDDR (p)
+#define SCM_PORT_WITH_PS_P(p)    SCM_TYP16_PREDICATE (scm_tc16_port_with_ps, p)
+#define SCM_PORT_WITH_PS_PORT(p) SCM_CAR (SCM_CELL_OBJECT_1 (p))
+#define SCM_PORT_WITH_PS_PS(p)   SCM_CDR (SCM_CELL_OBJECT_1 (p))
 
-#define SCM_COERCE_OUTPORT(p) (SCM_NIMP (p) && SCM_PORT_WITH_PS_P (p) \
-			       ? SCM_PORT_WITH_PS_PORT (p) \
-			       : p)
+#define SCM_COERCE_OUTPORT(p) \
+  (SCM_PORT_WITH_PS_P (p) ? SCM_PORT_WITH_PS_PORT (p) : p)
 
 #define SCM_PRINT_STATE_LAYOUT "sruwuwuwuwuwpwuwuwuruopr"
 typedef struct scm_print_state {
@@ -98,8 +97,7 @@ typedef struct scm_print_state {
 
 extern SCM scm_print_state_vtable;
 
-/* ? scm or long?  print.h and print.c disagree */
-extern long scm_tc16_port_with_ps;
+extern scm_t_bits scm_tc16_port_with_ps;
 
 extern SCM scm_print_options (SCM setting);
 SCM scm_make_print_state (void);
@@ -107,6 +105,7 @@ void scm_free_print_state (SCM print_state);
 extern void scm_intprint (long n, int radix, SCM port);
 extern void scm_ipruk (char *hdr, SCM ptr, SCM port);
 extern void scm_iprlist (char *hdr, SCM exp, int tlr, SCM port, scm_print_state *pstate);
+extern void scm_print_symbol_name (const char *str, size_t len, SCM port);
 extern void scm_prin1 (SCM exp, SCM port, int writingp);
 extern void scm_iprin1 (SCM exp, SCM port, scm_print_state *pstate);
 extern SCM scm_write (SCM obj, SCM port);
@@ -114,8 +113,7 @@ extern SCM scm_display (SCM obj, SCM port);
 extern SCM scm_simple_format (SCM port, SCM message, SCM args);
 extern SCM scm_newline (SCM port);
 extern SCM scm_write_char (SCM chr, SCM port);
-extern SCM scm_printer_apply (SCM proc, SCM exp, SCM port,
-                              scm_print_state *);
+extern SCM scm_printer_apply (SCM proc, SCM exp, SCM port, scm_print_state *);
 extern SCM scm_port_with_print_state (SCM port, SCM pstate);
 extern SCM scm_get_print_state (SCM port);
 extern int scm_valid_oport_value_p (SCM val);
@@ -124,7 +122,7 @@ extern void scm_init_print (void);
 #ifdef GUILE_DEBUG
 extern SCM scm_current_pstate (void);
 #endif 
-#endif  /* PRINTH */
+#endif  /* SCM_PRINT_H */
 
 /*
   Local Variables:
