@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996,1998,1999, 2000, 2002 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1998,1999,2000,2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -55,7 +55,9 @@
 #include "libguile/read.h"
 #include "libguile/root.h"
 #include "libguile/strings.h"
-#include "libguile/vectors.h"
+#include "libguile/modules.h"
+#include "libguile/validate.h"
+#include "libguile/deprecation.h"
 
 #include "libguile/strports.h"
 
@@ -77,10 +79,13 @@
    when rw_active is SCM_PORT_NEITHER.
 */
 
+scm_t_bits scm_tc16_strport;
+
+
 static int
 stfill_buffer (SCM port)
 {
-  scm_port *pt = SCM_PTAB_ENTRY (port);
+  scm_t_port *pt = SCM_PTAB_ENTRY (port);
   
   if (pt->read_pos >= pt->read_end)
     return EOF;
@@ -91,19 +96,25 @@ stfill_buffer (SCM port)
 /* change the size of a port's string to new_size.  this doesn't
    change read_buf_size.  */
 static void 
-st_resize_port (scm_port *pt, off_t new_size)
+st_resize_port (scm_t_port *pt, off_t new_size)
 {
-  SCM stream = SCM_PACK (pt->stream);
+  SCM old_stream = SCM_PACK (pt->stream);
+  SCM new_stream = scm_allocate_string (new_size);
+  unsigned long int old_size = SCM_STRING_LENGTH (old_stream);
+  unsigned long int min_size = min (old_size, new_size);
+  unsigned long int i;
 
   off_t index = pt->write_pos - pt->write_buf;
 
   pt->write_buf_size = new_size;
 
-  scm_vector_set_length_x (stream, SCM_MAKINUM (new_size));
+  for (i = 0; i != min_size; ++i)
+    SCM_STRING_CHARS (new_stream) [i] = SCM_STRING_CHARS (old_stream) [i];
 
-  /* reset buffer in case reallocation moved the string. */
+  /* reset buffer. */
   {
-    pt->read_buf = pt->write_buf = SCM_UCHARS (stream);
+    pt->stream = SCM_UNPACK (new_stream);
+    pt->read_buf = pt->write_buf = SCM_STRING_UCHARS (new_stream);
     pt->read_pos = pt->write_pos = pt->write_buf + index;
     pt->write_end = pt->write_buf + pt->write_buf_size;
     pt->read_end = pt->read_buf + pt->read_buf_size;
@@ -118,7 +129,7 @@ st_resize_port (scm_port *pt, off_t new_size)
 static void
 st_flush (SCM port)
 {
-  scm_port *pt = SCM_PTAB_ENTRY (port);
+  scm_t_port *pt = SCM_PTAB_ENTRY (port);
 
   if (pt->write_pos == pt->write_end)
     {
@@ -136,7 +147,7 @@ st_flush (SCM port)
 static void
 st_write (SCM port, const void *data, size_t size)
 {
-  scm_port *pt = SCM_PTAB_ENTRY (port);
+  scm_t_port *pt = SCM_PTAB_ENTRY (port);
   const char *input = (char *) data;
 
   while (size > 0)
@@ -144,7 +155,7 @@ st_write (SCM port, const void *data, size_t size)
       int space = pt->write_end - pt->write_pos;
       int write_len = (size > space) ? space : size;
       
-      strncpy ((char *) pt->write_pos, input, write_len);
+      memcpy ((char *) pt->write_pos, input, write_len);
       pt->write_pos += write_len;
       size -= write_len;
       input += write_len;
@@ -156,7 +167,7 @@ st_write (SCM port, const void *data, size_t size)
 static void
 st_end_input (SCM port, int offset)
 {
-  scm_port *pt = SCM_PTAB_ENTRY (port);
+  scm_t_port *pt = SCM_PTAB_ENTRY (port);
   
   if (pt->read_pos - pt->read_buf < offset)
     scm_misc_error ("st_end_input", "negative position", SCM_EOL);
@@ -168,7 +179,7 @@ st_end_input (SCM port, int offset)
 static off_t
 st_seek (SCM port, off_t offset, int whence)
 {
-  scm_port *pt = SCM_PTAB_ENTRY (port);
+  scm_t_port *pt = SCM_PTAB_ENTRY (port);
   off_t target;
 
   if (pt->rw_active == SCM_PORT_READ && offset == 0 && whence == SEEK_CUR)
@@ -240,7 +251,7 @@ st_seek (SCM port, off_t offset, int whence)
 static void
 st_truncate (SCM port, off_t length)
 {
-  scm_port *pt = SCM_PTAB_ENTRY (port);
+  scm_t_port *pt = SCM_PTAB_ENTRY (port);
 
   if (length > pt->write_buf_size)
     st_resize_port (pt, length);
@@ -258,12 +269,12 @@ SCM
 scm_mkstrport (SCM pos, SCM str, long modes, const char *caller)
 {
   SCM z;
-  scm_port *pt;
-  int str_len;
+  scm_t_port *pt;
+  size_t str_len;
 
   SCM_ASSERT (SCM_INUMP(pos) && SCM_INUM(pos) >= 0, pos, SCM_ARG1, caller);
-  SCM_ASSERT (SCM_ROSTRINGP(str), str, SCM_ARG1, caller);
-  str_len = SCM_ROLENGTH (str);
+  SCM_ASSERT (SCM_STRINGP (str), str, SCM_ARG1, caller);
+  str_len = SCM_STRING_LENGTH (str);
   if (SCM_INUM (pos) > str_len)
     scm_out_of_range (caller, pos);
   if (!((modes & SCM_WRTNG) || (modes & SCM_RDNG)))
@@ -274,7 +285,7 @@ scm_mkstrport (SCM pos, SCM str, long modes, const char *caller)
   SCM_SET_CELL_TYPE (z, scm_tc16_strport | modes);
   SCM_SETPTAB_ENTRY (z, pt);
   SCM_SETSTREAM (z, SCM_UNPACK (str));
-  pt->write_buf = pt->read_buf = SCM_ROUCHARS (str);
+  pt->write_buf = pt->read_buf = SCM_STRING_UCHARS (str);
   pt->read_pos = pt->write_pos = pt->read_buf + SCM_INUM (pos);
   pt->write_buf_size = pt->read_buf_size = str_len;
   pt->write_end = pt->read_end = pt->read_buf + pt->read_buf_size;
@@ -292,13 +303,51 @@ scm_mkstrport (SCM pos, SCM str, long modes, const char *caller)
 /* create a new string from a string port's buffer.  */
 SCM scm_strport_to_string (SCM port)
 {
-  scm_port *pt = SCM_PTAB_ENTRY (port);
+  scm_t_port *pt = SCM_PTAB_ENTRY (port);
+  SCM str;
 
   if (pt->rw_active == SCM_PORT_WRITE)
     st_flush (port);
-  return scm_makfromstr ((char *) pt->read_buf, pt->read_buf_size, 0);
+
+  str = scm_mem2string ((char *) pt->read_buf, pt->read_buf_size);
+  scm_remember_upto_here_1 (port);
+  return str;
 }
 
+SCM_DEFINE (scm_object_to_string, "object->string", 1, 1, 0,
+	    (SCM obj, SCM printer),
+	    "Return a Scheme string obtained by printing @var{obj}.\n"
+	    "Printing function can be specified by the optional second\n"
+	    "argument @var{printer} (default: @code{write}).")
+#define FUNC_NAME s_scm_object_to_string
+{
+  SCM str, port;
+
+  if (!SCM_UNBNDP (printer))
+    SCM_VALIDATE_PROC (2, printer);
+
+  str = scm_allocate_string (0);
+  port = scm_mkstrport (SCM_INUM0, str, SCM_OPN | SCM_WRTNG, FUNC_NAME);
+
+  if (SCM_UNBNDP (printer))
+    scm_write (obj, port);
+  else
+    scm_call_2 (printer, obj, port);
+
+  return scm_strport_to_string (port);
+}
+#undef FUNC_NAME
+
+#if (SCM_DEBUG_DEPRECATED == 0)
+
+SCM
+scm_strprint_obj (SCM obj)
+{
+  return scm_object_to_string (obj, SCM_UNDEFINED);
+}
+
+#endif /* (SCM_DEBUG_DEPRECATED == 0) */
+
 SCM_DEFINE (scm_call_with_output_string, "call-with-output-string", 1, 0, 0, 
            (SCM proc),
 	    "Calls the one-argument procedure @var{proc} with a newly created output\n"
@@ -312,58 +361,78 @@ SCM_DEFINE (scm_call_with_output_string, "call-with-output-string", 1, 0, 0,
 		     scm_make_string (SCM_INUM0, SCM_UNDEFINED),
 		     SCM_OPN | SCM_WRTNG,
                      FUNC_NAME);
-  scm_apply (proc, p, scm_listofnull);
+  scm_call_1 (proc, p);
 
-  return scm_strport_to_string (p);
+  return scm_get_output_string (p);
 }
 #undef FUNC_NAME
 
-
-
-/* Return a Scheme string obtained by printing a given object.
- */
-
-
-SCM
-scm_strprint_obj (SCM obj)
-{
-  SCM str;
-  SCM port;
-
-  str = scm_makstr (0, 0);
-  port = scm_mkstrport (SCM_INUM0, str, SCM_OPN | SCM_WRTNG, "scm_strprint_obj");
-  scm_prin1 (obj, port, 1);
-  {
-    return scm_strport_to_string (port);
-  }
-}
-
-
-
-
 SCM_DEFINE (scm_call_with_input_string, "call-with-input-string", 2, 0, 0,
-           (SCM str, SCM proc),
-	    "Calls the one-argument procedure @var{proc} with a newly created input\n"
-	    "port from which @var{string}'s contents may be read.  The value yielded\n"
-	    "by the @var{proc} is returned.")
+           (SCM string, SCM proc),
+	    "Calls the one-argument procedure @var{proc} with a newly\n"
+	    "created input port from which @var{string}'s contents may be\n"
+	    "read.  The value yielded by the @var{proc} is returned.")
 #define FUNC_NAME s_scm_call_with_input_string
 {
-  SCM p = scm_mkstrport(SCM_INUM0, str, SCM_OPN | SCM_RDNG, FUNC_NAME);
-  return scm_apply (proc, p, scm_listofnull);
+  SCM p = scm_mkstrport(SCM_INUM0, string, SCM_OPN | SCM_RDNG, FUNC_NAME);
+  return scm_call_1 (proc, p);
 }
 #undef FUNC_NAME
 
+SCM_DEFINE (scm_open_input_string, "open-input-string", 1, 0, 0,
+	    (SCM str),
+	    "Take a string and return an input port that delivers characters\n"
+	    "from the string. The port can be closed by\n"
+	    "@code{close-input-port}, though its storage will be reclaimed\n"
+	    "by the garbage collector if it becomes inaccessible.")
+#define FUNC_NAME s_scm_open_input_string
+{
+  SCM p = scm_mkstrport(SCM_INUM0, str, SCM_OPN | SCM_RDNG, FUNC_NAME);
+  return p;
+}
+#undef FUNC_NAME
+
+SCM_DEFINE (scm_open_output_string, "open-output-string", 0, 0, 0, 
+	    (void),
+	    "Return an output port that will accumulate characters for\n"
+	    "retrieval by @code{get-output-string}. The port can be closed\n"
+	    "by the procedure @code{close-output-port}, though its storage\n"
+	    "will be reclaimed by the garbage collector if it becomes\n"
+	    "inaccessible.")
+#define FUNC_NAME s_scm_open_output_string
+{
+  SCM p;
+
+  p = scm_mkstrport (SCM_INUM0, 
+		     scm_make_string (SCM_INUM0, SCM_UNDEFINED),
+		     SCM_OPN | SCM_WRTNG,
+                     FUNC_NAME);
+  return p;
+}
+#undef FUNC_NAME
+
+SCM_DEFINE (scm_get_output_string, "get-output-string", 1, 0, 0, 
+	    (SCM port),
+	    "Given an output port created by @code{open-output-string},\n"
+	    "return a string consisting of the characters that have been\n"
+	    "output to the port so far.")
+#define FUNC_NAME s_scm_get_output_string
+{
+  SCM_VALIDATE_OPOUTSTRPORT (1, port);
+  return scm_strport_to_string (port);
+}
+#undef FUNC_NAME
 
 
 /* Given a null-terminated string EXPR containing a Scheme expression
    read it, and return it as an SCM value. */
 SCM
-scm_read_0str (char *expr)
+scm_c_read_string (const char *expr)
 {
   SCM port = scm_mkstrport (SCM_INUM0,
 			    scm_makfrom0str (expr),
 			    SCM_OPN | SCM_RDNG,
-			    "scm_eval_0str");
+			    "scm_c_read_string");
   SCM form;
 
   /* Read expressions from that port; ignore the values.  */
@@ -376,26 +445,43 @@ scm_read_0str (char *expr)
 /* Given a null-terminated string EXPR containing Scheme program text,
    evaluate it, and return the result of the last expression evaluated.  */
 SCM
-scm_eval_0str (const char *expr)
+scm_c_eval_string (const char *expr)
 {
   return scm_eval_string (scm_makfrom0str (expr));
 }
 
+#if SCM_DEBUG_DEPRECATED == 0
 
-SCM_DEFINE (scm_eval_string, "eval-string", 1, 0, 0, 
-            (SCM string),
-	    "Evaluate @var{string} as the text representation of a Scheme form\n"
-	    "or forms, and return whatever value they produce.")
-#define FUNC_NAME s_scm_eval_string
+SCM
+scm_read_0str (char *expr)
 {
-  SCM port = scm_mkstrport (SCM_INUM0, string, SCM_OPN | SCM_RDNG,
-			    "scm_eval_0str");
+  scm_c_issue_deprecation_warning 
+    ("scm_read_0str is deprecated.  Use scm_c_read_string instead.");
+
+  return scm_c_read_string (expr);
+}
+
+SCM
+scm_eval_0str (const char *expr)
+{
+  scm_c_issue_deprecation_warning 
+    ("scm_eval_0str is deprecated.  Use scm_c_eval_string instead.");
+
+  return scm_c_eval_string (expr);
+}
+
+#endif
+
+static SCM
+inner_eval_string (void *data)
+{
+  SCM port = (SCM)data;
   SCM form;
   SCM ans = SCM_UNSPECIFIED;
 
   /* Read expressions from that port; ignore the values.  */
   while (!SCM_EOF_OBJECT_P (form = scm_read (port)))
-    ans = scm_eval_x (form);
+    ans = scm_primitive_eval_x (form);
 
   /* Don't close the port here; if we re-enter this function via a
      continuation, then the next time we enter it, we'll get an error.
@@ -404,24 +490,41 @@ SCM_DEFINE (scm_eval_string, "eval-string", 1, 0, 0,
 
   return ans;
 }
+
+SCM_DEFINE (scm_eval_string, "eval-string", 1, 0, 0, 
+            (SCM string),
+	    "Evaluate @var{string} as the text representation of a Scheme\n"
+	    "form or forms, and return whatever value they produce.\n"
+	    "Evaluation takes place in the environment returned by the\n"
+	    "procedure @code{interaction-environment}.")
+#define FUNC_NAME s_scm_eval_string
+{
+  SCM port = scm_mkstrport (SCM_INUM0, string, SCM_OPN | SCM_RDNG,
+			    "eval-string");
+  return scm_c_call_with_current_module (scm_interaction_environment (),
+					 inner_eval_string, (void *)port);
+}
 #undef FUNC_NAME
 
-void scm_make_stptob (void); /* Called from ports.c */
-
-void
+static scm_t_bits
 scm_make_stptob ()
 {
-  long tc = scm_make_port_type ("string", stfill_buffer, st_write);
+  scm_t_bits tc = scm_make_port_type ("string", stfill_buffer, st_write);
+
   scm_set_port_mark        (tc, scm_markstream);
   scm_set_port_end_input   (tc, st_end_input);
   scm_set_port_flush       (tc, st_flush);
   scm_set_port_seek        (tc, st_seek);
   scm_set_port_truncate    (tc, st_truncate);
+
+  return tc;
 }
 
 void
 scm_init_strports ()
 {
+  scm_tc16_strport = scm_make_stptob ();
+
 #include "libguile/strports.x"
 }
 
