@@ -1,8 +1,8 @@
 /* classes: h_files */
 
-#ifndef TAGSH
-#define TAGSH
-/* Copyright (C) 1995, 96, 97, 98, 99, 2000, 2002 Free Software Foundation, Inc.
+#ifndef SCM_TAGS_H
+#define SCM_TAGS_H
+/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001 Free Software Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,37 +50,49 @@
  ** It is here that tag bits are assigned for various purposes.
  **/
 
-
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
 
-/* #define SCM_VOIDP_TEST    */
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+
+
 
 /* In the beginning was the Word:
  */
-typedef long scm_bits_t;
+#ifdef HAVE_UINTPTR_T
+typedef uintptr_t scm_t_bits;
+typedef intptr_t scm_t_signed_bits;
+#else
+typedef unsigned long scm_t_bits;
+typedef signed long scm_t_signed_bits;
+#endif
 
 /* But as external interface, we use SCM, which may, according to the desired
  * level of type checking, be defined in several ways:
  */
-#if (SCM_DEBUG_TYPING_STRICTNESS == 1)
-    typedef union { struct { scm_bits_t n; } n; } SCM;
-    static SCM scm_pack(scm_bits_t b) { SCM s; s.n.n = b; return s; }
+#if (SCM_DEBUG_TYPING_STRICTNESS == 2)
+    typedef union { struct { scm_t_bits n; } n; } SCM;
+    static SCM scm_pack(scm_t_bits b) { SCM s; s.n.n = b; return s; }
 #   define SCM_UNPACK(x) ((x).n.n)
-#   define SCM_PACK(x) (scm_pack ((scm_bits_t) (x)))
-#elif defined (SCM_VOIDP_TEST)
+#   define SCM_PACK(x) (scm_pack ((scm_t_bits) (x)))
+#elif (SCM_DEBUG_TYPING_STRICTNESS == 1)
 /* This is the default, which provides an intermediate level of compile time
  * type checking while still resulting in very efficient code.
  */
-    typedef void * SCM;
-#   define SCM_UNPACK(x) ((scm_bits_t) (x))
+    typedef struct scm_unused_struct * SCM;
+#   define SCM_UNPACK(x) ((scm_t_bits) (x))
 #   define SCM_PACK(x) ((SCM) (x))
 #else
 /* This should be used as a fall back solution for machines on which casting
  * to a pointer may lead to loss of bit information, e. g. in the three least
  * significant bits.
  */
-    typedef scm_bits_t SCM;
+    typedef scm_t_bits SCM;
 #   define SCM_UNPACK(x) (x)
-#   define SCM_PACK(x) ((scm_bits_t) (x))
+#   define SCM_PACK(x) ((scm_t_bits) (x))
 #endif
 
 
@@ -208,7 +220,7 @@ typedef long scm_bits_t;
  * gloc    ..........SCM vcell..........001  ...........SCM cdr.............G
  * struct  ..........void * type........001  ...........void * data.........G
  * closure ..........SCM code...........011  ...........SCM env.............G
- * tc7	   .........long length....Gxxxx1S1  ..........void *data............
+ * tc7	   ......24.bits of data...Gxxxx1S1  ..........void *data............
  *
  *
  *
@@ -217,19 +229,11 @@ typedef long scm_bits_t;
  *		tc7_tags are 7 bit tags ending in 1x1.  These tags
  *		occur only in the CAR of heap cells, and have the
  *		handy property that all bits of the CAR above the
- *		bottom eight can be used to store a length, thus
+ *		bottom eight can be used to store some data, thus
  *		saving a word in the body itself.  Thus, we use them
- *		for strings, symbols, and vectors (among other
- *		things).
+ *		for strings and vectors (among other things).
  *
- *		SCM_LENGTH returns the bits in "length" (see the diagram).
- *		SCM_CHARS returns the data cast to "char *"
- *		SCM_CDR returns the data cast to "SCM"
- *		TYP7(X) returns bits 0...6 of SCM_CAR (X)
- *
- *		For the interpretation of SCM_LENGTH and SCM_CHARS
- *		that applies to a particular type, see the header file
- *		for that type.
+ *		TYP7(X) returns bits 0...6 of CELL_TYPE (X)
  *
  *              Sometimes we choose the bottom seven bits carefully,
  *              so that the 2-valued bit (called S bit) can be masked
@@ -255,7 +259,6 @@ typedef long scm_bits_t;
  *
  *		TYP16
  *		TYP16S
- *		GCTYP16
  *
  *		TYP16S functions similarly wrt to TYP16 as TYP7S to TYP7,
  *		but a different option bit is used (bit 2 for TYP7S,
@@ -278,10 +281,7 @@ typedef long scm_bits_t;
  * stored in the SCM_CAR of a non-immediate object have a 1 in bit 1:
  */
 
-#define SCM_SLOPPY_CONSP(x)  ((1 & SCM_CELL_TYPE (x)) == 0)
-#define SCM_SLOPPY_NCONSP(x) (!SCM_SLOPPY_CONSP(x))
-
-#define SCM_CONSP(x)  (!SCM_IMP (x) && SCM_SLOPPY_CONSP (x))
+#define SCM_CONSP(x)  (!SCM_IMP (x) && ((1 & SCM_CELL_TYPE (x)) == 0))
 #define SCM_NCONSP(x) (!SCM_CONSP (x))
 
 
@@ -290,18 +290,16 @@ typedef long scm_bits_t;
  */
 #define SCM_ECONSP(x) \
   (!SCM_IMP (x) \
-   && (SCM_SLOPPY_CONSP (x) \
+   && (SCM_CONSP (x) \
        || (SCM_TYP3 (x) == 1 \
 	   && (SCM_STRUCT_VTABLE_DATA (x)[scm_vtable_index_vcell] != 0))))
 #define SCM_NECONSP(x) (!SCM_ECONSP (x))
 
 
 
-#define SCM_CELLP(x) 	(((sizeof (scm_cell) - 1) & SCM_UNPACK (x)) == 0)
+#define SCM_CELLP(x) 	(((sizeof (scm_t_cell) - 1) & SCM_UNPACK (x)) == 0)
 #define SCM_NCELLP(x) 	(!SCM_CELLP (x))
 
-#define SCM_DOUBLE_CELLP(x)  (((2 * sizeof (scm_cell) - 1) & SCM_UNPACK (x)) == 0)
-
 /* See numbers.h for macros relating to immediate integers.
  */
 
@@ -322,32 +320,20 @@ typedef long scm_bits_t;
  */
 
 
+#define SCM_ITAG7(x) 		(127 & SCM_UNPACK (x))
 #define SCM_TYP7(x) 		(0x7f &        SCM_CELL_TYPE (x))
 #define SCM_TYP7S(x) 		((0x7f & ~2) & SCM_CELL_TYPE (x))
 
 
 #define SCM_TYP16(x) 		(0xffff & SCM_CELL_TYPE (x))
 #define SCM_TYP16S(x) 		(0xfeff & SCM_CELL_TYPE (x))
-#define SCM_GCTYP16(x) 		(0xff7f & SCM_CELL_TYPE (x))
-
-
-
-/* Testing and Changing GC Marks in Various Standard Positions
- */
-#define SCM_GCCDR(x)		SCM_PACK(~1L & SCM_UNPACK (SCM_CDR (x)))
-#define SCM_GCMARKP(x) 		(1 & SCM_UNPACK (SCM_CDR (x)))
-#define SCM_GC8MARKP(x) 	(0x80 & SCM_CELL_TYPE (x))
-#define SCM_SETGCMARK(x) 	SCM_SETOR_CDR (x, 1)
-#define SCM_CLRGCMARK(x) 	SCM_SETAND_CDR (x, ~1L)
-#define SCM_SETGC8MARK(x) 	SCM_SETOR_CAR (x, 0x80)
-#define SCM_CLRGC8MARK(x) 	SCM_SETAND_CAR (x, ~0x80L)
 
+#define SCM_TYP16_PREDICATE(tag,x) (SCM_NIMP (x) && SCM_TYP16 (x) == (tag))
 
 
 
-/* couple */
-#define scm_tc7_ssymbol		5
-#define scm_tc7_msymbol		7
+#define scm_tc7_symbol		5
+/* free                         7 */
 
 /* couple */
 #define scm_tc7_vector		13
@@ -362,11 +348,11 @@ typedef long scm_bits_t;
  * of these 7 bit tags!
  */
 #define scm_tc7_pws		31
-#define scm_tc7_lvector		39
 
 #ifdef HAVE_ARRAYS
 #define scm_tc7_llvect          29
 #define scm_tc7_uvect		37
+/* free                         39 */
 #define scm_tc7_fvect		45
 #define scm_tc7_dvect		47
 #define scm_tc7_cvect		53
@@ -376,7 +362,7 @@ typedef long scm_bits_t;
 #define scm_tc7_ivect		79
 #endif
 
-#define scm_tc7_contin		61
+/* free         		61 */
 #define scm_tc7_cclo		63
 #define scm_tc7_rpsubr		69
 #define scm_tc7_subr_0		85
@@ -391,16 +377,10 @@ typedef long scm_bits_t;
 #define scm_tc7_lsubr		119
 
 
-/* There are 256 port subtypes.  Here are the first few.
- * These must agree with the init function in ports.c
+/* There are 256 port subtypes.
  */
 #define scm_tc7_port		125
 
-#define scm_tc16_fport 		(scm_tc7_port + 0 * 256L)
-/* scm_tc16_pipe was here.  */
-#define scm_tc16_strport	(scm_tc7_port + 2 * 256L)
-#define scm_tc16_sfport 	(scm_tc7_port + 3 * 256L)
-
 
 /* There are 256 smob subtypes.  Here are the first four.
  */
@@ -415,21 +395,13 @@ typedef long scm_bits_t;
 /* scm_tc_free_cell is also the 0th smob type.  We place this
  * in free cells to tell the conservative marker not to trace it.
  */
-#define scm_tc_free_cell	0x007f
+#define scm_tc_free_cell	(scm_tc7_smob + 0 * 256L)
 
-/* Smob type 1 (note the dependency on the predicate SCM_NUMP)
+/* Smob type 1 to 3 (note the dependency on the predicate SCM_NUMP)
  */
-#define scm_tc16_big		0x017f
-
-/* Smob types 2 and 3:
- */
-#define scm_tc16_real           0x027f
-#define scm_tc16_complex        0x037f
-
-/* Smob type 4 allocated, but not initialized cells;
-   this is required to prevent the gc from hosing your cells if
-   you have to allocate while creating the cell*/
-#define scm_tc16_allocated	0x047f
+#define scm_tc16_big		(scm_tc7_smob + 1 * 256L)
+#define scm_tc16_real           (scm_tc7_smob + 2 * 256L)
+#define scm_tc16_complex        (scm_tc7_smob + 3 * 256L)
 
 
 /* {Immediate Values}
@@ -508,6 +480,7 @@ extern char *scm_isymnames[];   /* defined in print.c */
 #define SCM_IM_BIND		SCM_MAKISYM (31)
 
 #define SCM_IM_DELAY		SCM_MAKISYM (32)
+#define SCM_IM_CALL_WITH_VALUES SCM_MAKISYM (33)
 
 /* When a variable is unbound this is marked by the SCM_UNDEFINED
  * value.  The following is an unbound value which can be handled on
@@ -518,7 +491,7 @@ extern char *scm_isymnames[];   /* defined in print.c */
  * used instead.  It is not ideal to let this kind of unique and
  * strange values loose on the Scheme level.
  */
-#define SCM_UNBOUND		SCM_MAKIFLAG (33)
+#define SCM_UNBOUND		SCM_MAKIFLAG (34)
 
 #define SCM_UNBNDP(x)		(SCM_EQ_P ((x), SCM_UNDEFINED))
 
@@ -568,20 +541,20 @@ extern char *scm_isymnames[];   /* defined in print.c */
  case scm_tc7_subr_3:case scm_tc7_subr_2:case scm_tc7_rpsubr:case scm_tc7_subr_1o:\
  case scm_tc7_subr_2o:case scm_tc7_lsubr_2:case scm_tc7_lsubr
 
-#define scm_tcs_symbols scm_tc7_ssymbol:case scm_tc7_msymbol
-
 
 
 #if (SCM_DEBUG_DEPRECATED == 0)
 
-#define scm_tc16_flo		scm_tc16_real
-#define scm_tc_flo		0x017fL
-#define scm_tc_dblr		scm_tc16_real
-#define scm_tc_dblc		scm_tc16_complex
+#define SCM_SLOPPY_CONSP(x)  ((1 & SCM_CELL_TYPE (x)) == 0)
+#define SCM_SLOPPY_NCONSP(x) (!SCM_SLOPPY_CONSP(x))
+
+#define scm_tc7_ssymbol		scm_tc7_symbol
+#define scm_tc7_msymbol		scm_tc7_symbol
+#define scm_tcs_symbols         scm_tc7_symbol
 
 #endif  /* SCM_DEBUG_DEPRECATED == 0 */
 
-#endif  /* TAGSH */
+#endif  /* SCM_TAGS_H */
 
 /*
   Local Variables:
