@@ -311,8 +311,8 @@ uvec_equalp (SCM a, SCM b)
     result = SCM_BOOL_F;
   else if(SCM_UVEC_LENGTH (a) != SCM_UVEC_LENGTH (b))
     result = SCM_BOOL_F;
-  else if(memcmp(SCM_UVEC_BASE (a), SCM_UVEC_BASE (b),
-		 SCM_UVEC_LENGTH (a) * uvec_sizes[SCM_UVEC_TYPE(a)]) != 0)
+  else if(memcmp(SCM_UVEC_BASE (a), SCM_UVEC_BASE (b), SCM_UVEC_LENGTH (a))
+          != 0)
     result = SCM_BOOL_F;
 
   scm_remember_upto_here_2 (a, b);
@@ -331,12 +331,13 @@ uvec_free (SCM uvec)
 /* Utility procedures.                                              */
 /* ================================================================ */
 
+
 /* Create a new, uninitialized homogeneous numeric vector of type TYPE
    with space for LEN elements.  */
 inline static SCM
 make_uvec (const int pos, const char * func_name, int type, size_t len)
 {
-  if (len > (((size_t)-1) >> uvec_shifts[type]))
+  if (len > ((size_t) SIZE_MAX >> uvec_shifts[type]))
   {
     if(pos < 0)
       scm_out_of_range (func_name, scm_size2num (len)); 
