@@ -1,5 +1,5 @@
 ;;;; test-suite/lib.scm --- generic support for testing
-;;;; Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+;;;; Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
 ;;;;
 ;;;; This program is free software; you can redistribute it and/or modify
 ;;;; it under the terms of the GNU General Public License as published by
@@ -17,10 +17,9 @@
 ;;;; Boston, MA 02111-1307 USA
 
 (define-module (test-suite lib)
-  ;;:use-module (ice-9 stack-catch)
-  :use-module (ice-9 regex))
-
-(export
+  :use-module (ice-9 stack-catch)
+  :use-module (ice-9 regex)
+  :export (
 
  ;; Exceptions which are commonly being tested for.
  exception:out-of-range exception:unbound-var
@@ -40,12 +39,8 @@
  make-log-reporter
  full-reporter
  user-reporter
- format-test-name
+ format-test-name))
 
- ;; The more modern way to check exceptions is to use `run-test-exception',
- ;; but that uses (ice-9 stack-catch), which is not available yet, so we use
- ;; these two.  We also prevent `run-test-exception' definition below.
- signals-error? signals-error?*)
 
 ;;;; If you're using Emacs's Scheme mode:
 ;;;;   (put 'with-test-prefix 'scheme-indent-function 1)
@@ -286,7 +281,7 @@
   `(run-test ,name #f (lambda () ,body ,@rest)))
 
 ;;; A helper function to implement the macros that test for exceptions.
-'(define (run-test-exception name exception expect-pass thunk)
+(define (run-test-exception name exception expect-pass thunk)
   (run-test name expect-pass
     (lambda ()
       (stack-catch (car exception)
@@ -487,29 +482,3 @@
       (apply full-reporter result name args)))
 
 (set! default-reporter full-reporter)
-
-
-;;;; Detecting whether errors occur
-
-;;; (signals-error? KEY BODY ...)
-;;; Evaluate the expressions BODY ... .  If any errors occur, return #t;
-;;; otherwise, return #f.
-;;;
-;;; KEY indicates the sort of errors to look for; it can be a symbol,
-;;; indicating that only errors with that name should be caught, or
-;;; #t, meaning that any kind of error should be caught.
-(defmacro signals-error? key-and-body
-  `(signals-error?* ,(car key-and-body)
-		    (lambda () ,@(cdr key-and-body))))
-
-;;; (signals-error?* KEY THUNK)
-;;; Apply THUNK, catching errors.  If any errors occur, return #t;
-;;; otherwise, return #f.
-;;;
-;;; KEY indicates the sort of errors to look for; it can be a symbol,
-;;; indicating that only errors with that name should be caught, or
-;;; #t, meaning that any kind of error should be caught.
-(define (signals-error?* key thunk)
-  (catch key
-	 (lambda () (thunk) #f)
-	 (lambda args #t)))
